@@ -1,9 +1,10 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
+from __future__ import annotations
+
 from .auth_by_plugin import AuthByPlugin
 from .network import ID_TOKEN_AUTHENTICATOR
 
@@ -20,6 +21,7 @@ class AuthByIdToken(AuthByPlugin):
 
     def __init__(self, id_token):
         """Initialized an instance with an IdToken."""
+        super().__init__()
         self._id_token = id_token
 
     def authenticate(self, authenticator, service_name, account, user, password):
@@ -28,5 +30,5 @@ class AuthByIdToken(AuthByPlugin):
 
     def update_body(self, body):
         """Idtoken needs the authenticator and token attributes set."""
-        body[u"data"][u"AUTHENTICATOR"] = ID_TOKEN_AUTHENTICATOR
-        body[u"data"][u"TOKEN"] = self._id_token
+        body["data"]["AUTHENTICATOR"] = ID_TOKEN_AUTHENTICATOR
+        body["data"]["TOKEN"] = self._id_token
